#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>

#include "pollin.h"
#include "spi.h"
#include "rf12.h"

void rf12_init(void) {
	spi_init();
	_delay_ms(250); // Give POR some time (Power-On-Reset)
	
	spi_transferWord(0xC0E0);			// AVR CLK: 10MHz
	spi_transferWord(0x80D7);			// Enable FIFO
	spi_transferWord(0xC2AB);			// Data Filter: internal
	spi_transferWord(0xCA81);			// Set FIFO mode
	spi_transferWord(0xE000);			// disable wakeuptimer
	spi_transferWord(0xC800);			// disable low duty cycle
	spi_transferWord(0xC4F7);			// AFC settings: autotuning: -10kHz...+7,5kHz
}

void rf12_setbandwidth(unsigned char bandwidth, unsigned char gain, unsigned char drssi) {
	spi_transferWord(0x9400|((bandwidth&7)<<5)|((gain&3)<<3)|(drssi&7));
}

void rf12_setfreq(unsigned short freq)
{	if (freq<96)				// 430,2400MHz
		freq=96;
	else if (freq>3903)			// 439,7575MHz
		freq=3903;
	spi_transferWord(0xA000|freq);
}

void rf12_setbaud(unsigned short baud)
{
	if (baud<663)
		return;
	if (baud<5400)					// Baudrate= 344827,58621/(R+1)/(1+CS*7)
		spi_transferWord(0xC680|((43104/baud)-1));
	else
		spi_transferWord(0xC600|((344828UL/baud)-1));
}

void rf12_setpower(unsigned char power, unsigned char mod)
{	
	spi_transferWord(0x9800|(power&7)|((mod&15)<<4));
}

void rf12_txdata(unsigned char *data, unsigned char number){
	unsigned char i;
	spi_transferWord(0x8238);			// TX on
	//_delay_ms(10.0);
	rf12_ready();
	spi_transferWord(0xB8AA);
	rf12_ready();
	spi_transferWord(0xB8AA);
	rf12_ready();
	spi_transferWord(0xB8AA);
	rf12_ready();
	spi_transferWord(0xB82D);
	rf12_ready();
	spi_transferWord(0xB8D4);
	for (i=0; i<number; i++)
	{		rf12_ready();
		spi_transferWord(0xB800|(*data++));
	}
	rf12_ready();
	spi_transferWord(0x8208);			// TX off
}

void rf12_ready(void) {	
	LED_AN(LED2);
	CHIPSELECT();
	while(!(PIN_SPI & MISO));
}

void rf12_rxenable(void){
	spi_transferWord(0x82C8);			// RX on
	spi_transferWord(0xCA81);			// set FIFO mode
	spi_transferWord(0xCA83);			// enable FIFO	

	//spi_transferWord(0xCA85);			// set FIFO mode
	//spi_transferWord(0xCA87);			// enable FIFO	without synchron pattern
}

void rf12_rxdisable(void){
	spi_transferWord(0x8208);			// RX off
}

unsigned char rf12_rxbyte(){
	return spi_transferWord(0xB000);
}
