#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>

#include "spi.h"
#include "uart.h"

/*
	SPI Bezeichnungen
	MOSI  = MasterOutSlaveIn
	MISO  = MasterInSlaveOut
	SCK   = Clocksignal
	SS/CS = SlaveSelect / ChipSelect
	
	Relevante Register:
	SPCR  - Spi control register
	 +--SPIE SPI Interrupt Enable
	 +--SPE  SPI Enable
	 \--MSTR Master/Slave Select (1=Master)

	SPSR  - Spi Status Register
	 +--SPIF SPI Interrupt flag - Set after transfer is complete
	 +--
	 
	SPDR  - Spi Data Register
	 The SPI Data Register is a Read/Write Register used for data transfer between the Register File
	 and the SPI Shift Register. Writing to the register initiates data transmission. Reading the regis-
	 er causes the Shift Register Receive buffer to be read.

*/

void spi_wait_transfer_complete(void) {
	while(!(SPSR & (1<<SPIF))) {	}
}


/*
 * Initialisiert die SPI Schnittstelle
 */
void spi_init(void) {
	// MOSI,SCK und CS als Output markieren
	DDR_SPI |= (1<<MOSI) | (1<<SCK) | (1<<CS);
	//      Enable     Master    1/16 F_CPU
	SPCR = (1<<SPE) | (1<<MSTR)| (1<<SPR0) ;
}


/*
 * Transfers two bytes per SPI to the Chip
 */
unsigned short spi_transferWord(unsigned short message){
	// Vor dem Senden ChipSelect auf LOW ziehen
	CHIPSELECT();
	//_delay_us(1.0);

	// Daten versenden	
	// Wenn mehr als 1 Byte verschickt werden soll:
	if (message > 0xff) {
		SPDR=((message & 0xff00)>>8);
		spi_wait_transfer_complete();
		// Erster Teil versandt, erstes Byte auf 00000000 setzen
		message &= 0x00FF;
		
		// SPDR Inhalt in erstes Byte schreiben
		message |= (SPDR<<8);
	} else {
		message &= 0x00FF;
	}

	SPDR=(message & 0x00ff);
	spi_wait_transfer_complete();
	
	// Zweites Byte leeren
	message &=0xFF00;
	
	message |= (0x00FF & SPDR);
	
	//_delay_us(1.0);
	
	// Nach dem Senden ChipSelect wieder auf HIGH ziehen
	CHIPDESELECT();
	//_delay_us(1.0);
	
	// Auslesen des Registers mit den Daten und Rückgabe dieser
	return message;
}


