#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>

#include "uart.h"

// Code aus dem Beispielskript für das Pollin Eval.-Board
// ( http://www.mikrocontroller.net/articles/Pollin_Funk-AVR-Evaluationsboard#8-Bit_Z.C3.A4hler_mit_RS232-Anschluss )

/*
 * Initialisiert den UART mit Baudrate aus den Defines
 */
void UART_init(void){
    // Baudrate setzen
    // BAUDRATE Baud bei F_CPU
    UBRRH = (uint8_t) ((F_CPU / (16 * BAUDRATE) - 1) >> 8);
    UBRRL = (uint8_t)  (F_CPU / (16 * BAUDRATE) - 1);

    //      Empfangen,   Senden erlauben
    UCSRB = (1<<RXEN) | (1<<TXEN);

    // Frame Format:              8 Bits,                  No Parity,          1 Stopbit
    UCSRC = (1<<URSEL) | ((1<<UCSZ1) | (1<<UCSZ0)) | ((0<<UPM1) | (0<<UPM0)) | (0<<USBS);
}

/*
 * Prüft ob am UART Zeichen vorliegen
 */
uint8_t UART_peekchar(void){
    // sofort zurückkehren und Zustand melden
    return UCSRA & (1<<RXC);
}

/*
 * Wartet auf Zeichen am UART und gibt diese zurück
 */
uint8_t UART_getchar(void){
    // Warte bis UART Eingang voll
    while ( !(UCSRA & (1<<RXC)) )
        ;
    return UDR;
}

/*
 * Zeichen in den UART schreiben
 */
void UART_putchar(char z){
    // Warte bis UART Ausgang frei
    while ( !(UCSRA & (1<<UDRE)) )
        ;
    UDR = z;
}
 
/*
 * Zeichenkette (nur String?) in den UART schreiben
 */
void UART_puts(char *s){
    while ( *s )
    {
        UART_putchar(*s);
        s++;
    }
}
